@@ -1,5 +1,5 @@
 #   Copyright (C) 2006-2007 Daniel Burrows
-#   Copyright (C) 2020 Scott Hansen
+#   Copyright (C) 2021 Scott Hansen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -92,7 +92,8 @@ def splittext(text, search, attr):
 class URLChooser:
 
     def __init__(self, extractedurls, compact=False, reverse=False, nohelp=False, dedupe=False,
-                 shorten=True, run="", single=False, pipe=False, genconf=False, width=0):
+                 shorten=True, run="", runsafe="", single=False, pipe=False,
+                 genconf=False, width=0):
         self.conf = expanduser("~/.config/urlscan/config.json")
         self.keys = {'/': self._search_key,
                      '0': self._digits,
@@ -178,6 +179,7 @@ class URLChooser:
         self.shorten = shorten
         self.compact = compact
         self.run = run
+        self.runsafe = runsafe
         self.single = single
         self.pipe = pipe
         self.search = False
@@ -208,7 +210,9 @@ class URLChooser:
                        "/ - search :: "
                        "URL opening mode - {}")
         self.link_open_modes = ["Web Browser", "Xdg-Open"] if self.xdg is True else ["Web Browser"]
-        if self.run:
+        if self.runsafe:
+            self.link_open_modes.insert(0, self.runsafe)
+        elif self.run:
             self.link_open_modes.insert(0, self.run)
         self.nohelp = nohelp
         if nohelp is False:
@@ -323,8 +327,8 @@ class URLChooser:
 
     def _open_url(self):
         """<Enter> or <space>"""
-        load_text = "Loading URL..." if self.link_open_modes[0] != self.run \
-            else "Executing: {}".format(self.run)
+        load_text = "Loading URL..." if self.link_open_modes[0] != (self.run or self.runsafe) \
+            else "Executing: {}".format(self.run or self.runsafe)
         if os.environ.get('BROWSER') not in ['elinks', 'links', 'w3m', 'lynx']:
             self._footer_display(load_text, 5)
 
@@ -462,7 +466,7 @@ class URLChooser:
     def _reverse(self):
         """ R """
         # Reverse items
-        fpo = self.top.body.focus_position
+        fpo = self.top.base_widget.body.focus_position
         if self.compact is True:
             self.items.reverse()
         else:
@@ -475,8 +479,8 @@ class URLChooser:
                 else:
                     rev.insert(2, item)
             self.items = rev
-        self.top.body = urwid.ListBox(self.items)
-        self.top.body.focus_position = self._cur_focus(fpo)
+        self.top.base_widget.body = urwid.ListBox(self.items)
+        self.top.base_widget.body.focus_position = self._cur_focus(fpo)
 
     def _context(self):
         """ c """
@@ -505,7 +509,7 @@ class URLChooser:
         cmds = COPY_COMMANDS_PRIMARY if pri else COPY_COMMANDS
         for cmd in cmds:
             try:
-                proc = Popen(shlex.split(cmd), stdin=PIPE)
+                proc = Popen(shlex.split(cmd), stdin=PIPE, stdout=DEVNULL, stderr=DEVNULL)
                 proc.communicate(input=url.encode(sys.getdefaultencoding()))
                 self._footer_display("Copied url to {} selection".format(
                     "primary" if pri is True else "clipboard"), 5)
@@ -635,7 +639,7 @@ class URLChooser:
 
     def _link_handler(self):
         """Function to cycle through opening links via webbrowser module,
-        xdg-open or custom expression passed with --run.
+        xdg-open or custom expression passed with --run-safe or --run.
 
         """
         mode = self.link_open_modes.pop()
@@ -659,10 +663,17 @@ class URLChooser:
                     self.search = False
                     self.enter = False
             elif self.link_open_modes[0] == "Web Browser":
-                webbrowser.open(url)
+                webbrowser.open(url.replace('&', '\&'))
             elif self.link_open_modes[0] == "Xdg-Open":
                 run = 'xdg-open "{}"'.format(url)
                 process = Popen(shlex.split(run), stdout=PIPE, stdin=PIPE)
+            elif self.link_open_modes[0] == self.runsafe:
+                if self.pipe:
+                    process = Popen(shlex.split(self.runsafe), stdout=PIPE, stdin=PIPE)
+                    process.communicate(input=url.encode(sys.getdefaultencoding()))
+                else:
+                    cmd = [i.format(url) for i in shlex.split(self.runsafe)]
+                    Popen(cmd).communicate()
             elif self.link_open_modes[0] == self.run and self.pipe:
                 process = Popen(shlex.split(self.run), stdout=PIPE, stdin=PIPE)
                 process.communicate(input=url.encode(sys.getdefaultencoding()))
